@@ -140,7 +140,7 @@ class LightgbmAlgorithm(BaseAlgorithm):
 
 lgbm_bin_params = {
     "objective": ["binary"],
-    "metric": ["binary_logloss"],
+    "metric": ["binary_logloss", "auc"],
     "num_leaves": [3, 7, 15, 31],  # , 63, 127],
     "learning_rate": [0.05, 0.075, 0.1, 0.15],
     "feature_fraction": [0.8, 0.9, 1.0],
@@ -190,7 +190,7 @@ classification_multi_default_params = {
 
 lgbr_params = copy.deepcopy(lgbm_bin_params)
 lgbr_params["objective"] = ["regression"]
-lgbr_params["metric"] = ["l2"]
+lgbr_params["metric"] = ["rmse"]
 
 AlgorithmsRegistry.add(
     BINARY_CLASSIFICATION,
@@ -221,7 +221,7 @@ regression_required_preprocessing = [
 
 regression_default_params = {
     "objective": "regression",
-    "metric": "l2",
+    "metric": "rmse",
     "num_leaves": 15,
     "learning_rate": 0.1,
     "feature_fraction": 0.9,
